@@ -143,14 +143,17 @@ class VariablesVisitor(AsyncVisitor):
 
 
 class BlockVariableVisitor(AsyncVisitor):
-    def __init__(self, source: str, position: Optional[Position] = None, in_args: bool = True) -> None:
+    def __init__(
+        self, namespace: Namespace, source: str, position: Optional[Position] = None, in_args: bool = True
+    ) -> None:
         super().__init__()
-
+        self.namespace = namespace
         self.source = source
         self.position = position
         self.in_args = in_args
 
         self._results: Dict[str, VariableDefinition] = {}
+        self.current_kw_doc: Optional[KeywordDoc] = None
 
     async def get(self, model: ast.AST) -> List[VariableDefinition]:
 
@@ -164,15 +167,26 @@ class BlockVariableVisitor(AsyncVisitor):
         if self.position is None or self.position >= range_from_node(node).start:
             return await super().visit(node)
 
+    async def visit_Keyword(self, node: ast.AST) -> None:  # noqa: N802
+        try:
+            await self.generic_visit(node)
+        finally:
+            self.current_kw_doc = None
+
     async def visit_KeywordName(self, node: ast.AST) -> None:  # noqa: N802
         from robot.parsing.lexer.tokens import Token as RobotToken
         from robot.parsing.model.statements import KeywordName
         from robot.variables.search import VariableSearcher
 
+        from ..parts.model_helper import ModelHelperMixin
+
         n = cast(KeywordName, node)
         name_token = cast(Token, n.get_token(RobotToken.KEYWORD_NAME))
 
         if name_token is not None and name_token.value:
+            keyword = await ModelHelperMixin.get_keyword_definition_at_token(self.namespace, name_token)
+            self.current_kw_doc = keyword
+
             for variable_token in filter(
                 lambda e: e.type == RobotToken.VARIABLE,
                 tokenize_variables(name_token, identifiers="$", ignore_errors=True),
@@ -192,6 +206,7 @@ class BlockVariableVisitor(AsyncVisitor):
                         end_line_no=variable_token.lineno,
                         end_col_offset=variable_token.end_col_offset,
                         source=self.source,
+                        keyword_doc=self.current_kw_doc,
                     )
 
     def get_variable_token(self, token: Token) -> Optional[Token]:
@@ -221,7 +236,7 @@ class BlockVariableVisitor(AsyncVisitor):
             try:
                 argument = self.get_variable_token(argument_token)
 
-                if argument is not None:
+                if argument is not None and argument.value != "@{}":
                     if (
                         self.in_args
                         and self.position is not None
@@ -240,6 +255,7 @@ class BlockVariableVisitor(AsyncVisitor):
                             end_line_no=argument.lineno,
                             end_col_offset=argument.end_col_offset,
                             source=self.source,
+                            keyword_doc=self.current_kw_doc,
                         )
 
             except VariableError:
@@ -816,7 +832,10 @@ class Namespace:
             *[
                 (
                     await BlockVariableVisitor(
-                        self.source, position, isinstance(test_or_keyword_nodes[-1], Arguments) if nodes else False
+                        self,
+                        self.source,
+                        position,
+                        isinstance(test_or_keyword_nodes[-1], Arguments) if nodes else False,
                     ).get(test_or_keyword)
                 )
                 if test_or_keyword is not None
@@ -890,6 +909,7 @@ class Namespace:
         top_level: bool = False,
         variables: Optional[Dict[str, Any]] = None,
         source: Optional[str] = None,
+        parent_import: Optional[Import] = None,
     ) -> None:
         async def _import(
             value: Import, variables: Optional[Dict[str, Any]] = None
@@ -923,27 +943,46 @@ class Namespace:
                     if value.name is None:
                         raise NameSpaceError("Resource setting requires value.")
 
-                    result = await self._get_resource_entry(value.name, base_dir, sentinel=value, variables=variables)
-                    result.import_range = value.range()
-                    result.import_source = value.source
+                    source = await self.imports_manager.find_resource(value.name, base_dir, variables=variables)
 
-                    self._import_entries[value] = result
-                    if result.variables:
-                        variables = None
-
-                    if top_level and (
-                        not result.library_doc.errors
-                        and top_level
-                        and not result.imports
-                        and not result.variables
-                        and not result.library_doc.keywords
-                    ):
-                        await self.append_diagnostics(
-                            range=value.range(),
-                            message=f"Imported resource file '{value.name}' is empty.",
-                            severity=DiagnosticSeverity.WARNING,
-                            source=DIAGNOSTICS_SOURCE_NAME,
+                    if self.source == source:
+                        if parent_import:
+                            await self.append_diagnostics(
+                                range=parent_import.range(),
+                                message="Possible circular import.",
+                                severity=DiagnosticSeverity.INFORMATION,
+                                source=DIAGNOSTICS_SOURCE_NAME,
+                                related_information=[
+                                    DiagnosticRelatedInformation(
+                                        location=Location(str(Uri.from_path(value.source)), value.range()),
+                                        message=f"'{Path(self.source).name}' is also imported here.",
+                                    )
+                                ],
+                            )
+                    else:
+                        result = await self._get_resource_entry(
+                            value.name, base_dir, sentinel=value, variables=variables
                         )
+                        result.import_range = value.range()
+                        result.import_source = value.source
+
+                        self._import_entries[value] = result
+                        if result.variables:
+                            variables = None
+
+                        if top_level and (
+                            not result.library_doc.errors
+                            and top_level
+                            and not result.imports
+                            and not result.variables
+                            and not result.library_doc.keywords
+                        ):
+                            await self.append_diagnostics(
+                                range=value.range(),
+                                message=f"Imported resource file '{value.name}' is empty.",
+                                severity=DiagnosticSeverity.WARNING,
+                                source=DIAGNOSTICS_SOURCE_NAME,
+                            )
 
                 elif isinstance(value, VariablesImport):
 
@@ -1060,6 +1099,7 @@ class Namespace:
                                     top_level=False,
                                     variables=variables,
                                     source=entry.library_doc.source,
+                                    parent_import=imp if top_level else parent_import,
                                 )
                             except (asyncio.CancelledError, SystemExit, KeyboardInterrupt):
                                 raise
@@ -1085,8 +1125,6 @@ class Namespace:
                                     already_imported_resources is not None
                                     and already_imported_resources.library_doc.source
                                 ):
-                                    # self._resources[entry.import_name] = entry
-
                                     await self.append_diagnostics(
                                         range=entry.import_range,
                                         message=f"Resource {entry} already imported.",
