@@ -98,7 +98,7 @@ class RobotLanguageServerProtocol(LanguageServerProtocol):
     short_name = "RobotCode"
     version = __version__
 
-    file_extensions = {"robot", "resource", "py"}
+    file_extensions = {"robot", "resource", "py", "yaml", "yml"}
 
     def __init__(self, server: "RobotLanguageServer"):
         super().__init__(server)
