@@ -575,9 +575,16 @@ class Lock:
         return f"<{res[1:-1]} [{extra}]>"
 
     @asynccontextmanager
-    async def __inner_lock(self) -> AsyncGenerator[bool, None]:
-        with self._lock as r:
-            yield r
+    async def __inner_lock(self) -> AsyncGenerator[Any, None]:
+        b = self._lock.acquire(blocking=False)
+        while not b:
+            await asyncio.sleep(0)
+            b = self._lock.acquire(blocking=False)
+
+        try:
+            yield None
+        finally:
+            self._lock.release()
 
     async def acquire(self) -> bool:
         async with self.__inner_lock():
@@ -585,51 +592,54 @@ class Lock:
                 self._locked = True
                 return True
 
-        if self._waiters is None:
-            self._waiters = deque()
+            if self._waiters is None:
+                self._waiters = deque()
 
-        fut = create_sub_future()
-
-        with self._lock:
+            fut = create_sub_future()
             self._waiters.append(fut)
 
         try:
             try:
                 await fut
             finally:
-                self._waiters.remove(fut)
+                async with self.__inner_lock():
+                    self._waiters.remove(fut)
+                    self._locked = True
         except asyncio.CancelledError:
             async with self.__inner_lock():
-                if not self._locked:
-                    self._wake_up_first()
+                if self._locked:
+                    await self._wake_up_first()
             raise
 
-        async with self.__inner_lock():
-            self._locked = True
-
         return True
 
     async def release(self) -> None:
-        if self._locked:
-            async with self.__inner_lock():
+        async with self.__inner_lock():
+            if self._locked:
                 self._locked = False
-            self._wake_up_first()
-        else:
-            raise RuntimeError("Lock is not acquired.")
+            else:
+                raise RuntimeError("Lock is not acquired.")
 
-    def _wake_up_first(self) -> None:
+            await self._wake_up_first()
+
+    async def _wake_up_first(self) -> None:
         if not self._waiters:
             return
+
         try:
             fut = next(iter(self._waiters))
         except StopIteration:
             return
 
-        if not fut.done():
-            if fut._loop == asyncio.get_running_loop():
+        def s() -> None:
+            if not fut.done():
                 fut.set_result(True)
-            else:
-                fut._loop.call_soon_threadsafe(fut.set_result, True)
+
+        if fut._loop == asyncio.get_running_loop():
+            if not fut.done():
+                fut.set_result(True)
+        else:
+            fut._loop.call_soon_threadsafe(s)
 
 
 class FutureInfo:
